###Supplementary material methods 2. Bioinformatic Analyses
####López-García, Á., Jurado-Rivera, J.A., Bota, J., Cifre, J., Baraza, E. 
####Space and vine cultivar interact to determine the arbuscular mycorrhizal fungal community 
####composition. Journal of Fungi 2020.

###Adpated from https://benjjneb.github.io/dada2/tutorial_1_8.html [1]

#Loading packages----
library(dada2)
library(phyloseq)
library(ggplot2)
library(lulu)
library(dplyr)

#Preparing files----
#Set working directory, locate fastq files in a subfolder called "reads"
setwd("d:/Usuario/Desktop/NGS Mallorca")

#File where the sequences are and we read in the names of the fastq files
path <- "d:/Usuario/Desktop/NGS Mallorca/reads"
list.files(path)

#Get forward and reverse fastq filenames
fnFs <- sort(list.files(path, pattern="_R1_001.fastq", full.names = TRUE))
fnRs <- sort(list.files(path, pattern="_R2_001.fastq", full.names = TRUE))

#Extract sample names
sample.names <- sapply(strsplit(basename(fnFs), "_"), `[`, 1)

#Quality check of random samples----
plotQualityProfile(fnFs[c(1,3,12,22,30)]) #FORWARD
plotQualityProfile(fnRs[c(1,3,12,22,30)]) #REVERSE

#Filter and Trim----
#Place filtered files in filtered/ subdirectory
filtFs <- file.path("d:/Usuario/Desktop/NGS Mallorca/reads", "filtered", paste0(sample.names, "_F_filt.fastq.gz"))
filtRs <- file.path("d:/Usuario/Desktop/NGS Mallorca/reads", "filtered", paste0(sample.names, "_R_filt.fastq.gz"))

#Use standard filtering parameters: maxN=0 (DADA2 requires no Ns), truncQ=2, rm.phix=TRUE and maxEE=2.
#Use truncLen to truncate samples where observed low quality (see Quality check section).
#Use trimLeft to removed the primers NS31-AML2.
out <- filterAndTrim(fnFs, filtFs, fnRs, filtRs, truncLen=c(295,290),trimLeft = c(22,20),
                     maxN=0, maxEE=c(2,2), truncQ=2, rm.phix=TRUE,
                     compress=TRUE, multithread=FALSE) # On Windows set multithread=FALSE
head(out)

#Learn error rates----
errF <- learnErrors(filtFs, multithread=TRUE) 
errR <- learnErrors(filtRs, multithread=TRUE) 

#Check at least visually the estimated error rates
plotErrors(errF, nominalQ=TRUE)
plotErrors(errR, nominalQ=TRUE)

#Dereplication----
derepFs <- derepFastq(filtFs, verbose=TRUE)
derepRs <- derepFastq(filtRs, verbose=TRUE)

#Name the derep-class objects by the sample names
names(derepFs) <- sample.names
names(derepRs) <- sample.names

#Sample inference----
dadaFs <- dada(derepFs, err=errF, multithread=TRUE)
dadaRs <- dada(derepRs, err=errR, multithread=TRUE)

#Inspecting the returned dada-class object:

dadaFs[[1]] #first forward sample
dadaFs[[2]] #second forward sample

dadaRs[[1]] #first reverse sample
dadaRs[[2]] #second reverse sample

#Merge paired reads----
mergers <- mergePairs(dadaFs, derepFs, dadaRs, derepRs, verbose=TRUE)

#Inspect the merger data.frame from the first sample
head(mergers[[1]])

#Sequence table----
seqtab <- makeSequenceTable(mergers)
dim(seqtab)

# Inspect distribution of sequence lengths
table(nchar(getSequences(seqtab)))

#Remove chimeric sequences----
seqtab.nochim <- removeBimeraDenovo(seqtab, method="consensus", multithread=TRUE, verbose=TRUE)
dim(seqtab.nochim)

#Calculate amount of non-chimeric sequences
sum(seqtab.nochim)/sum(seqtab)

#Track reads through the pipeline----
getN <- function(x) sum(getUniques(x))
track <- cbind(out, sapply(dadaFs, getN), sapply(dadaRs, getN), sapply(mergers, getN), rowSums(seqtab.nochim))
colnames(track) <- c("input", "filtered", "denoisedF", "denoisedR", "merged", "nonchim")
rownames(track) <- sample.names
head(track)
write.table(track, file="track.txt")

#Getting a fasta file----
secuencias<-colnames(seqtab.nochim)

#Assign new rownames for 860 ASVs
de0a9<-paste (rep (0, 9), c(1:9), sep="")
de0a9<-paste (rep (0, 9), de0a9, sep="")
de0a9<-paste (rep (0, 9), de0a9, sep="")
de10a99<-paste (rep (0, 90), c(10:99), sep="")
de10a99<-paste (rep (0, 90), de10a99, sep="")
de100a860<-paste (rep (0, 760), c(100:860), sep="")
seqs<-c(de0a9,de10a99,de100a860)
sequence_names <- paste0("ASV",seqs)

#Extract otus (sequences)
pwd <- getwd()
setwd(pwd)
main_path <- pwd

sinkname <- file.path(main_path,"Mallorca.ASVs.fasta")
{
  sink(sinkname)
  for (seqX in seq.int(secuencias)) {
    header <- paste0(">",sequence_names[seqX],"\n")
    cat(header)
    seqq <- paste0(secuencias[seqX],"\n")
    cat(seqq)
  }
  sink()
}

#Taxonomic assignment----
taxa <- assignTaxonomy(seqtab.nochim, "silva_nr_v132_train_set_MaarjAM.fas", multithread=TRUE)
taxa<-cbind(taxa,sequence_names)

#Remove non-Glomeromycotan fungi
Glomeromycetes<-which(taxa[,3]=="Glomeromycetes")
ASV_AMF<-seqtab.nochim[,Glomeromycetes]
taxa_AMF<-taxa[Glomeromycetes,]
colnames(ASV_AMF)<-taxa_AMF[,7]

#Extract otus (sequences)
pwd <- getwd()
setwd(pwd)
main_path <- pwd

secuencias<-rownames(taxa_AMF)
sequence_names_AMF<-colnames(ASV_AMF)

sinkname <- file.path(main_path,"ASV_AMF.otus")
{
  sink(sinkname)
  for (seqX in seq.int(secuencias)) {
    header <- paste0(">",sequence_names_AMF[seqX],"\n")
    cat(header)
    seqq <- paste0(secuencias[seqX],"\n")
    cat(seqq)
  }
  sink()
}


#Implementing LULU correction----
#First, obtain a matchlist table using vsearch
system("vsearch --usearch_global ASV_AMF.otus --db d:/Usuario/Desktop/NGSMallorca/ASV_AMF.otus --self --id .84 --iddef 1 --userout match_list.txt -userfields query+target+id --maxaccepts 0 --query_cov .9 --maxhits 10")

#edit manually the obtained match list to fit into a two columns table and load it
match_list<-read.table("match_list.txt",header=F)

otu_table<-t(ASV_AMF)
write.table(otu_table,file="OTUid.txt") 
#manually modify the table to include "OTUid" as head of the first column and load it again.
otu_table<-read.table(file = "OTUid.txt", header=TRUE,as.is=TRUE, row.names = 1)

#Run LULU
curated_result <- lulu(otu_table,matchlist)
ASV_AMF_LULU<-curated_result$curated_table
ASV_AMF_LULU<-t(ASV_AMF_LULU)

#Obtain a fasta file for LULU representative ASVs
secuencias_lulu<-colnames(ASV_AMF_LULU) #nombres OTUs
secuencias_lulu2<-gsub("ASV", "", secuencias_lulu) #quitar palabra OTU, solo numero

secuencias_lulu2<-as.numeric(secuencias_lulu2)
uniques<-rownames(taxa)
uniques<-uniques[secuencias_lulu2]

#Extract OTUs (sequences) to fasta file.
pwd <- getwd()
setwd(pwd)
main_path <- pwd

sinkname <- file.path(main_path,"ASV_AMF_LULU.otus")
{
  sink(sinkname)
  for (seqX in seq.int(ncol(ASV_AMF_LULU))) {
    header <- paste0(">",colnames(ASV_AMF_LULU)[seqX],"\n")
    cat(header)
    seqq <- paste0(uniques[seqX],"\n")
    cat(seqq)
  }
  sink()
}


#Obtain taxa table for LULU ASVs
taxa_ASV_AMF_LULU<-taxa[secuencias_lulu2,]

#Blasta against MaarjAM to name VTs----
#Run vsearch to cluster at 97%
system("vsearch -usearch_global ASV_AMF_LULU.otus.fas -db silva_nr_v132_train_set_MaarjAM.fas -iddef 0 -id 0.97 -userout output.txt -userfields query+target+id0+id1+id2+mism+ql+alnlen+caln -maxaccepts 0 -maxrejects 0 -top_hits_only -query_cov 0.9 -maxhits 1 -output_no_hits -matched matched97_maarjAM.fasta -notmatched NotMatched97_maarjAM.fasta -blast6out blast_maarjAM_97_output.txt")

#13 ASVs did not reach 97% with any MaarjAM VT. Group them between them. Create a fasta file with them: "clustering.fasta"
system("vsearch --cluster_size clustering.fasta --id 0.97 --sizein --sizeout --uc clusters --centroids clusters_97 --clusters VT")

#Manually add a column with the new VT identity to the taxa table: "taxa_ASV_AMF_LULU_Blast_info.txt"
taxa_ASV_AMF_LULU_VT_Blast_info<-read.table("taxa_ASV_AMF_LULU_Blast_info.txt", header=T)

#Merge the same VT into a unique taxonomic unit
VTs<-c(levels(taxa_ASV_AMF_LULU_VT_Blast_info$VT))

#VT_table
r<-rownames(ASV_AMF_LULU) #samples
c<- VTs
rc<- list(r,c) #list of both names
ASV_AMF_LULU_VT<-matrix(nrow=65,ncol=58,dimnames=rc) #empty matrix 
ASV_AMF_LULU_VT<-as.matrix(ASV_AMF_LULU_VT)

for (i in 1:length(VTs)){
  if(length(which(taxa_ASV_AMF_LULU_VT_Blast_info$VT==VTs[i]))>1){
    ASV_AMF_LULU_VT[,i]<-rowSums(ASV_AMF_LULU[,which(taxa_ASV_AMF_LULU_VT_Blast_info$VT==VTs[i])])
  } else {
    ASV_AMF_LULU_VT[,i]<-ASV_AMF_LULU[,which(taxa_ASV_AMF_LULU_VT_Blast_info$VT==VTs[i])]
  }
}

save(ASV_AMF_LULU_VT, file="ASV_AMF_LULU_VT.RData")


